@@ -27,7 +27,7 @@ from aptdaemon.enums import *
 from aptdaemon.gtk3widgets import AptProgressDialog
 
 from LanguageSelector.LanguageSelector import *
-from LanguageSelector.ImSwitch import ImSwitch
+from LanguageSelector.ImConfig import ImConfig
 from LanguageSelector.macros import *
 from LanguageSelector.utils import language2locale
 from LanguageSelector.LangCache import ExceptionPkgCacheBroken
@@ -187,12 +187,9 @@ class GtkLanguageSelector(LanguageSelectorBase):
         combo.pack_start(cell, True)
         combo.add_attribute(cell, 'text', IM_NAME)
         combo.set_model(model)
-        self.imSwitch = ImSwitch()
+        self.ImConfig = ImConfig()
         self._blockSignals = False
 
-        # remove dangling ImSwitch symlinks if present
-        self.imSwitch.removeDanglingSymlinks()
-
         # build the treeview
         self.setupLanguageTreeView()
         if self.is_admin:
@@ -225,7 +222,7 @@ class GtkLanguageSelector(LanguageSelectorBase):
             if not self._cache.havePackageLists:
                 d = Gtk.MessageDialog(parent=self.window_main,
                                       flags=Gtk.DialogFlags.MODAL,
-                                      type=Gtk.MessageType.INFO,
+                                      message_type=Gtk.MessageType.INFO,
                                       buttons=Gtk.ButtonsType.CANCEL)
                 d.set_markup("<big><b>%s</b></big>\n\n%s" % (
                     _("No language information available"),
@@ -246,7 +243,7 @@ class GtkLanguageSelector(LanguageSelectorBase):
             if self.options.verify_installed:
                 self.verifyInstalledLangPacks()
 
-        if not self.imSwitch.available():
+        if not self.ImConfig.available():
             self.combobox_input_method.set_sensitive(False)
         self.setSensitive(True)
 
@@ -258,9 +255,6 @@ class GtkLanguageSelector(LanguageSelectorBase):
             raise AttributeError('No such widget: ' + name)
         return o
 
-    def run(self):
-        Gtk.main()
-
     def setSensitive(self, value):
         if value:
             self.window_main.set_sensitive(True)
@@ -438,34 +432,22 @@ class GtkLanguageSelector(LanguageSelectorBase):
 
     @blockSignals
     def check_input_methods(self):
-        """ check if the selected langauge has input method support
-            and set checkbutton_enable_input_methods accordingly
-        """
-        if not self.imSwitch.available():
+        if not self.ImConfig.available():
             return
-        # get the current first item in the user LANGUAGE list,
-        # but with country code added if not already present
-        loc = language2locale(self.userEnvLanguage)
-        code = re.split('[.@]', loc)[0]
 
         combo = self.combobox_input_method
-        #cell = combo.get_child().get_cell_renderers()[0]
-        # FIXME: use something else than a hardcoded value here
-        #cell.set_property("wrap-width",300)
-        #cell.set_property("wrap-mode",Pango.WRAP_WORD)
         model = combo.get_model()
         model.clear()
 
         # find the default
-        currentIM = self.imSwitch.getInputMethodForLocale(code)
-        if currentIM in (None, 'default'):
-            currentIM = 'none'
-        #print("Current IM: "+currentIM)
+        currentIM = self.ImConfig.getCurrentInputMethod()
 
         # find out about the other options
-        for (i, IM) in enumerate(self.imSwitch.getAvailableInputMethods()):
+        names = dict(default=_('default'), xim=_('none'), ibus='IBus',
+          scim='SCIM', hangul='Hangul', thai='Thai')
+        for (i, IM) in enumerate(self.ImConfig.getAvailableInputMethods()):
+            name = names[IM] if IM in names else IM
             iter = model.append()
-            name = _('none') if IM == 'none' else IM
             model.set_value(iter, IM_CHOICE, IM)
             model.set_value(iter, IM_NAME, name)
             if IM == currentIM:
@@ -530,7 +512,7 @@ class GtkLanguageSelector(LanguageSelectorBase):
     def error(self, summary, msg):
         d = Gtk.MessageDialog(parent=self.window_main,
                               flags=Gtk.DialogFlags.MODAL,
-                              type=Gtk.MessageType.ERROR,
+                              message_type=Gtk.MessageType.ERROR,
                               buttons=Gtk.ButtonsType.CLOSE)
         d.set_markup("<big><b>%s</b></big>\n\n%s" % (summary, msg))
         d.set_title=("")
@@ -674,7 +656,7 @@ class GtkLanguageSelector(LanguageSelectorBase):
             # FIXME: add "details"
             d = Gtk.MessageDialog(parent=self.window_main,
                                   flags=Gtk.DialogFlags.MODAL,
-                                  type=Gtk.MessageType.QUESTION)
+                                  message_type=Gtk.MessageType.QUESTION)
             d.set_markup("<big><b>%s</b></big>\n\n%s" % (
                 _("The language support is not installed completely"),
                 _("Some translations or writing aids available for your "
@@ -1005,21 +987,26 @@ class GtkLanguageSelector(LanguageSelectorBase):
     # window_main signal handlers                      #
     ####################################################
     def on_delete_event(self, event, data):
-        if self.window_main.get_property("sensitive") is False:
-            return True
-        else:
-            Gtk.main_quit()
+        app = self.window_main.get_application()
+        if app:
+            app.remove_window(self.window_main)
 
     def on_button_quit_clicked(self, widget):
-        Gtk.main_quit()
+        app = self.window_main.get_application()
+        if app:
+            app.remove_window(self.window_main)
 
     @honorBlockedSignals
     def on_window_main_key_press_event(self, widget, event):
         keyname = Gdk.keyval_name(event.keyval)
         if event.get_state() & Gdk.ModifierType.CONTROL_MASK and keyname == "w":
-            Gtk.main_quit()
-        if event.get_state() == 0 and keyname == "Escape":
-            Gtk.main_quit()
+            app = self.window_main.get_application()
+            if app:
+                app.remove_window(self.window_main)
+        if (event.get_state() | Gdk.ModifierType.MOD2_MASK) == Gdk.ModifierType.MOD2_MASK and keyname == "Escape":
+            app = self.window_main.get_application()
+            if app:
+                app.remove_window(self.window_main)
         return None
 
     ####################################################
@@ -1055,6 +1042,7 @@ class GtkLanguageSelector(LanguageSelectorBase):
         self.writeUserLanguage(envLanguage)
         self.userEnvLanguage = envLanguage
         self.check_input_methods()
+        self.updateLocaleChooserCombo()
         #os.environ["LANGUAGE"]=envLanguage
 
 #    @honorBlockedSignals
@@ -1076,16 +1064,12 @@ class GtkLanguageSelector(LanguageSelectorBase):
 
     @honorBlockedSignals
     def on_combobox_input_method_changed(self, widget):
-        loc = language2locale(self.userEnvLanguage)
-        code = re.split('[.@]', loc)[0]
-
         combo = self.combobox_input_method
         model = combo.get_model()
         if combo.get_active() < 0:
             return
         (IM_choice, IM_name) = model[combo.get_active()]
-        #print("IM: "+IM_choice+"\t"+code)
-        self.imSwitch.setInputMethodForLocale(IM_choice, code)
+        self.ImConfig.setInputMethod(IM_choice)
 
     
     ####################################################
